package main

import (
	"os"
	"strings"

	"go.wit.com/dev/alexflint/arg"
	"go.wit.com/lib/protobuf/forgepb"
	"go.wit.com/lib/protobuf/gitpb"
	"go.wit.com/log"
)

// sent via -ldflags
var VERSION string
var BUILDTIME string

// used for shell auto completion
var ARGNAME string = "go-mod-clean"

var golangVersion string = "1.22"

var pp *arg.Parser
var forge *forgepb.Forge

// var check *gitpb.Repo
var configSave bool

func main() {
	var check *gitpb.Repo
	log.Info("go-mod-clean version", VERSION, "built on", BUILDTIME)
	pp = arg.MustParse(&argv)

	if argv.Bash {
		argv.doBash()
		os.Exit(0)
	}
	if len(argv.BashAuto) != 0 {
		argv.doBashAuto()
		os.Exit(0)
	}
	// load the ~/.config/forge/ config
	// this lets you configure repos you have read/write access too
	forge = forgepb.InitPB() // todo: make this scan only if pb file not found

	// figure out what directory we are running in
	check = findPwdRepo()
	if check == nil {
		log.Info("this directory isn't in a golang project (not in ~/go/src nor a go.work file)")
		badExit(nil, nil)
	}

	// deletes all the git notes
	if argv.Purge {
		purgeNotes(check)
		okExit(check, "notes gone")
	}

	if argv.Restore {
		// attempt to restore from ~/go/pkg/mod/
		if err := restoreFromGoPkg(check); err != nil {
			badExit(check, err)
		}
		okExit(check, "go.mod and go.sum restored from ~/go/pkg/mod/")
	}

	if argv.Strict {
		log.Info("Starting --strict mode")
		if err := doStrict(check); err != nil {
			badExit(check, err)
		}
		okExit(check, "go.mod seems clean")
	}
	if argv.Smart {
		// best effort
		if err := doSmart(check); err != nil {
			// badExit(check, err)
		}
		okExit(check, "maybe it's ok")
	}

	if argv.Force {
		if err := doForce(check); err != nil {
			badExit(check, err)
		}
	}

	okExit(check, "go.sum seems clean")
}

func findPwdRepo() *gitpb.Repo {
	var check *gitpb.Repo
	// attempt to use the working directory
	// this is probably what happens most of the time
	pwd, _ := os.Getwd()
	if strings.HasPrefix(pwd, forge.GetGoSrc()) {
		gopath := strings.TrimPrefix(pwd, forge.GetGoSrc())
		gopath = strings.Trim(gopath, "/")
		log.Info("findRepo() trying gopath", gopath)
		check = forge.FindByGoPath(gopath)
		if check != nil {
			log.Info("findRepo() worked", check.GetGoPath())
			return check
		}
	}
	return nil
}

func saveAsMetadata(repo *gitpb.Repo) error {
	cname := repo.GetCurrentBranchName()
	if repo.GetGoPrimitive() {
		if err := repo.AutogenSave([]string{"go.mod"}, cname, true); err != nil {
			return err
		}
	} else {
		if err := repo.AutogenSave([]string{"go.mod", "go.sum"}, cname, true); err != nil {
			return err
		}
	}
	return nil
}
